Data Structures and Algorithms
==============================
#### A (hopefully) comprehensive list of data structures and algorithms useful in programming contests. ####

Introduction
------------

The aim of this repository is to provide a central hub with robust implementations of the data structures and algorithms one might encounter during algorithmics programming contests.

Most of the inspiration comes from [here](http://www.infoarena.ro/arhiva-educationala). Another website with a similar goal is [here](http://e-maxx.ru/algo/).

Table of Contents
-----------------
#### Data structures
* Fenwick tree: [C++](/cpp/data_structures/fenwick_tree/fenwick_tree.cpp), [Java](/java/src/data_structures/fenwick_tree/FenwickTree.java)
* Segment tree: [C++](/cpp/data_structures/segment_tree/segment_tree.cpp), [Java](/java/src/data_structures/segment_tree/SegmentTree.java)
* Disjoint set: [C++](/cpp/data_structures/disjoint_set/disjoint_set.cpp), [Java](/java/src/data_structures/disjoint_set/DisjointSet.java)

#### Algorithms
##### Mathematics
* Euclidean algorithm for determining the greatest common divisor: [C++](/cpp/algorithms/euclid_gcd/euclid_gcd.cpp), [Java](/java/src/algorithms/euclid_gcd/GreatestCommonDivisor.java)
* Extended Euclidean algorithm: [C++](/cpp/algorithms/extended_euclid/extended_euclid.cpp), [Java](/java/src/algorithms/extended_euclid/ExtendedEuclid.java)
* Sieve of Eratosthenes: [C++](/cpp/algorithms/eratosthenes_sieve/eratosthenes_sieve.cpp), [Java](/java/src/algorithms/eratosthenes_sieve/EratosthenesSieve.java)

##### Dynamic Programming
* Longest common substring: [C++](/cpp/algorithms/longest_common_substring/longest_common_substring.cpp), [Java](/java/src/algorithms/longest_common_substring/LongestCommonSubstring.java)

##### Graph algorithms
* Basic algorithms:
    * Topological sort: [C++](/cpp/algorithms/topological_sort/topological_sort.cpp), [Java](/java/src/algorithms/topological_sort/TopologicalSort.java)
    * Depth First Search: [C++](/cpp/algorithms/depth_first_search/depth_first_search.cpp), [Java](/java/src/algorithms/depth_first_search/ConnectedComponents.java)
    * Breadth First Search: [C++](/cpp/algorithms/breadth_first_search/breadth_first_search.cpp), [Java](/java/src/algorithms/breadth_first_search/BreadthFirstSearch.java)

* Shortest path algorithms:
    * Roy-Floyd algorithm for determining the all-pairs shortest paths: [C++](/cpp/algorithms/roy_floyd/roy_floyd.cpp), [Java](/java/src/algorithms/roy_floyd/RoyFloyd.java)

##### String algorithms
* Pattern matching:
    * Knuth-Morris-Pratt algorithm: [C++](/cpp/algorithms/knuth_morris_pratt/knuth_morris_pratt.cpp), [Java](/java/src/algorithms/knuth_morris_pratt/KnuthMorrisPratt.java)

License
-------

[MIT License](http://opensource.org/licenses/mit-license.php)

&copy; 2013 Sebastian Claici &lt;sebastianclaici@gmail.com&gt;

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.


